@@ -16,6 +16,7 @@ from tornado.ioloop import TimeoutError
 
 from distributed.batched import BatchedStream
 from distributed.core import rpc, dumps, loads, connect, read, write
+from distributed.client import _wait
 from distributed.scheduler import Scheduler
 from distributed.sizeof import sizeof
 from distributed.worker import Worker, error_message, logger
@@ -474,7 +475,7 @@ def test_compute_stream(s, a, b):
     yield write(stream, {'op': 'close'})
 
 
-@gen_cluster(executor=True, ncores=[('127.0.0.1', 1)])
+@gen_cluster(client=True, ncores=[('127.0.0.1', 1)])
 def test_active_holds_tasks(e, s, w):
     future = e.submit(slowinc, 1, delay=0.2)
     yield gen.sleep(0.1)
@@ -494,3 +495,48 @@ def test_io_loop(loop):
     assert s.io_loop is loop
     w = Worker(s.ip, s.port, loop=loop)
     assert w.io_loop is loop
+
+
+@gen_cluster(client=True, ncores=[])
+def test_spill_to_disk(e, s):
+    np = pytest.importorskip('numpy')
+    w = Worker(s.ip, s.port, loop=s.loop, memory_limit=1000)
+    yield w._start()
+
+    x = e.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='x')
+    yield _wait(x)
+    y = e.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='y')
+    yield _wait(y)
+
+    assert set(w.data) == {x.key, y.key}
+    assert set(w.data.fast) == {x.key, y.key}
+
+    z = e.submit(np.random.randint, 0, 255, size=500, dtype='u1', key='z')
+    yield _wait(z)
+    assert set(w.data) == {x.key, y.key, z.key}
+    assert set(w.data.fast) == {y.key, z.key}
+    assert set(w.data.slow) == {x.key}
+
+    yield x._result()
+    assert set(w.data.fast) == {x.key, z.key}
+    assert set(w.data.slow) == {y.key}
+
+
+@gen_cluster(client=True)
+def test_access_key(c, s, a, b):
+    def f(i):
+        from distributed.worker import thread_state
+        return thread_state.key
+
+    futures = [c.submit(f, i, key='x-%d' % i) for i in range(20)]
+    results = yield c._gather(futures)
+    assert list(results) == ['x-%d' % i for i in range(20)]
+
+
+@gen_cluster(client=True)
+def test_run_dask_worker(c, s, a, b):
+    def f(dask_worker=None):
+        return dask_worker.id
+
+    response = yield c._run(f)
+    assert response == {a.address: a.id, b.address: b.id}
